from glob import glob
import numpy as np
from ase.visualize import view
from ase.io import read
from gpaw import GPAW, PoissonSolver
from gpaw.utilities import h2gpts
from gpaw.xc.libvdwxc import vdw_df

fnames = glob('xyz_displaced/s01_??.xyz')
fnames.sort()

configs = []

for fname in fnames:
    system = read(fname)
    configs.append(system)


atoms = configs[-1]
atoms.center(vacuum=5.0) # XXX 10 should be okay.

d1 = configs[1].positions[-1]
d2 = configs[2].positions[-1]
dx1 = d2 - d1
DX = configs[2].positions - configs[1].positions
#print DX

#print [np.vdot(dx1, DX[i]) for i, pos in enumerate(DX)]
i1 = [i for i, pos in enumerate(DX) if np.vdot(dx1, DX[i]) > 0]
i2 = [i for i, pos in enumerate(DX) if np.vdot(dx1, DX[i]) <= 0]

#print i1
#print i2

#raise SystemExit
mol1 = atoms[i1]
mol2 = atoms[i2]
assert len(mol1) + len(mol2) == len(atoms)

def getcalc(**kwargs):
    kwargs1 = dict(gpts=h2gpts(0.3, atoms.cell, idiv=8),
                   mode='lcao',
                   basis='dzp',
                   xc=vdw_df(),
                   poissonsolver=PoissonSolver(relax='GS', eps=1e-8))
    kwargs1.update(kwargs)
    calc = GPAW(**kwargs1)
    return calc

atoms.calc = getcalc(txt='gpaw.s22.txt')
mol1.calc = getcalc(txt='gpaw.mol1.txt')
mol2.calc = getcalc(txt='gpaw.mol2.txt')

for i, config in enumerate(configs):
    atoms.positions[:] = config.positions
    atoms.center()

    E = atoms.get_potential_energy()
    F0 = atoms.get_forces()

    mol1.positions[:] = atoms.positions[i1]
    mol2.positions[:] = atoms.positions[i2]
    Eref1 = mol1.get_potential_energy()
    Eref2 = mol2.get_potential_energy()
    Fref1 = mol1.get_forces()
    Fref2 = mol2.get_forces()

    x = np.linalg.norm(mol2.get_center_of_mass() - mol1.get_center_of_mass())
    #F = np.vdot(F0, DX / np.linalg.norm(DX))

    if atoms.calc.wfs.world.rank == 0:
        fd = open('results.%03d.txt' % i, 'w')
        fd.write('%15.10f %15.10f\n' % (x, E - Eref1 - Eref2))
        fd.flush()
